<?php
/**
 * 2007-2017 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2017 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!class_exists('EstoAPI')) {
    include_once _PS_MODULE_DIR_ . 'esto/classes/EstoAPI.php';
}

class Esto extends PaymentModule
{
    const MODE_LIVE = 'live';
    const MODE_TEST = 'test';
    const CREATE_ORDER_BEFORE = 'before';
    const CREATE_ORDER_AFTER = 'after';
    const COUNTRY_ESTONIA = 'estonia';
    const COUNTRY_LITHUANIA = 'lithuania';
    const COUNTRY_LATVIA = 'latvia';
    const COUNTRY_FINLAND = 'finland';
    const MIN_AMOUNT = 0.1;
    const ESTO_X_MIN_AMOUNT = 0.1;
    const MAX_AMOUNT = 10000;
    const API_URLS = [
        self::COUNTRY_ESTONIA => 'https://api.esto.ee/v2/',
        self::COUNTRY_LITHUANIA => 'https://api.estopay.lt/v2/',
        self::COUNTRY_LATVIA => 'https://api.esto.lv/v2/',
        # Primary Api country option 'Finland' exists in settings, but no such Esto Api exists, use EE Esto Api instead
        self::COUNTRY_FINLAND => 'https://api.esto.ee/v2/',
    ];
    const API_COUNTRIES = [
        self::COUNTRY_ESTONIA => 'EE',
        self::COUNTRY_LITHUANIA => 'LT',
        self::COUNTRY_LATVIA => 'LV',
        self::COUNTRY_FINLAND => 'FI',
    ];
    const ISO_COUNTRIES = [
        'EE' => self::COUNTRY_ESTONIA,
        'LV' => self::COUNTRY_LATVIA,
        'LT' => self::COUNTRY_LITHUANIA,
        'FI' => self::COUNTRY_FINLAND,
    ];
    const SUBSTITUTED_COUNTRIES = [
        'finland' => 'estonia',
    ];
    const CACHE_TIME = 10; // in minutes
    const CARD_TYPE_NAME = 'CARD';

    public $useOrderRefAsReference = false;

    public function __construct()
    {
        $this->name = 'esto';
        $this->tab = 'payments_gateways';
        $this->version = '1.6.7';
        $this->author = 'Mikk Mihkel Nurges, ESTO AS | Veebipoed.ee';
        $this->is_eu_compatible = 1;
        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
        $this->controllers = ['payment', 'validation'];
        $this->bootstrap = true;
        $this->currencies = false;

        parent::__construct();

        $this->displayName = $this->l('ESTO');
        $this->description = $this->l('Payment module for paying with ESTO lease');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        $this->useOrderRefAsReference = Configuration::get('ESTO_USE_ORDER_REF_AS_REFERENCE');
    }

    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        if (!$this->registrationHook()) {
            return false;
        }

        $this->createOrderStates();
        $this->addUniqueConstraintToCartIdInOrdersTable();


        if (!Configuration::updateValue('ESTO_SHOP_ID_EE', '')
            || !Configuration::updateValue('ESTO_SECRET_KEY_EE', '')
            || !Configuration::updateValue('ESTO_SHOP_ID_LV', '')
            || !Configuration::updateValue('ESTO_SECRET_KEY_LV', '')
            || !Configuration::updateValue('ESTO_SHOP_ID_LT', '')
            || !Configuration::updateValue('ESTO_SECRET_KEY_LT', '')
            || !Configuration::updateValue('ESTO_CREATE_ORDER', self::CREATE_ORDER_AFTER)
            || !Configuration::updateValue('ESTO_COUNTRY', self::COUNTRY_ESTONIA)
            || !Configuration::updateValue('ESTO_SHOW_CHECKOUT_LOGO', true)
            || !Configuration::updateValue('ESTO_CONNECTION_MODE', self::MODE_LIVE)
            || !Configuration::updateValue('ESTO_MIN_ORDER_AMOUNT', self::MIN_AMOUNT)
            || !Configuration::updateValue('ESTO_MAX_ORDER_AMOUNT', self::MAX_AMOUNT)
            || !Configuration::updateValue('ESTO_X_MIN_ORDER_AMOUNT', self::ESTO_X_MIN_AMOUNT)
            || !Configuration::updateValue('ESTO_X_MAX_ORDER_AMOUNT', self::MAX_AMOUNT)
            || !Configuration::updateValue('ESTO_PAY_LATER_MIN_ORDER_AMOUNT', self::MIN_AMOUNT)
            || !Configuration::updateValue('ESTO_PAY_LATER_MAX_ORDER_AMOUNT', self::MAX_AMOUNT)
            || !Configuration::updateValue('ESTO_MONTHLY_ENABLED_enable_estomonthly', '')
            || !Configuration::updateValue('ESTO_MONTHLY_URL', '')
            || !Configuration::updateValue('ESTO_MONTHLY_DESCRIPTION', '')
            || !Configuration::updateValue('ESTO_MONTHLY_LOGO', '')
            || !Configuration::updateValue('ESTO_CHECKOUT_LOGO', '')
            || !Configuration::updateValue('ESTO_X_IS_ACTIVE', false)
            || !Configuration::updateValue('ESTO_X_CHECKOUT_LOGO', '')
            || !Configuration::updateValue('ESTO_PAY_LATER_IS_ACTIVE', false)
            || !Configuration::updateValue('ESTO_PAY_LATER_CHECKOUT_LOGO', '')
            || !Configuration::updateValue('ESTO_PAY_IS_ACTIVE', false)
            || !Configuration::updateValue('ESTO_PAY_CHECKOUT_LOGO', '')
            || !Configuration::updateValue('ESTO_USE_SPECIFIC_ENDPOINT_EE', false)
            || !Configuration::updateValue('ESTO_USE_SPECIFIC_ENDPOINT_LV', false)
            || !Configuration::updateValue('ESTO_USE_SPECIFIC_ENDPOINT_LT', false)
            || !Configuration::updateValue('ESTO_PAY_CARD_IS_ACTIVE', false)
            || !Configuration::updateValue('ESTO_PAY_CARD_CHECKOUT_LOGO', '')
            || !$this->createBanksTable()
        ) {
            return false;
        }

        return $this->installDefaultLanguageConfiguration();
    }

    private function installDefaultLanguageConfiguration()
    {
        $defaults = [
            # two logos below were set as custom logos, we need to keep these two files available on hdd
            # logo-esto-pay.png replaced by duplicate of esto_pay_logo_en.png (its not lang specific icon)
            # logo-esto-pay-later is kept as it was before, its identical to esto_pay_later_logo_en.png
            # Default payment option name and description are set elsewhere,
            # no need to set any custom name/description with predefined value here
//            'et' => [
//                'ESTO_PAY_CHECKOUT_LOGO' => 'logo-esto-pay.png',
//                'ESTO_PAY_LATER_CHECKOUT_LOGO' => 'logo-esto-pay-later.png',
//                'ESTO_PAY_DISPLAY_NAME' => 'Tasu turvaliselt ja mugavalt pangamaksega!',
//                'ESTO_PAY_DISPLAY_DESCRIPTION' => 'Tasu SEB, Swedbanki, LHV, Luminori, Coopi või Citadele pangamaksega või Revolutiga. Võimalik tasuda ka Läti, Leedu või Soome pangamaksega. Turvalised ja mugavad makseviisid toob teieni ESTO Pay.',
//            ],
        ];

        $languages = Language::getLanguages(true);

        $result = true;

        foreach($languages as $language) {
            $iso_code = $language['iso_code'];
            $id_lang = $language['id_lang'];

            if(!isset($defaults[$iso_code])) {
                continue;
            }

            $values = $defaults[$iso_code];

            foreach($values as $key => $value) {
                $result &= Configuration::updateValue($key, [$id_lang => $value]);
            }
        }

        return $result;
    }


    public function createBanksTable(){
        $db = Db::getInstance();
        $query = 'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'esto_banks
		(
            `id` int(11) NOT NULL auto_increment,
            `data` text,
            `date_update` timestamp DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
            PRIMARY KEY  (`id`)
		) ENGINE='.(defined('_MYSQL_ENGINE_')?_MYSQL_ENGINE_:"MyISAM").'  DEFAULT CHARSET=utf8;';

        $db->Execute($query);
        return true;
    }

    public function uninstallBanksTable(){
        Db::getInstance()->Execute('DROP TABLE IF EXISTS '._DB_PREFIX_.'esto_banks');
    }
  
    // To prevent double orders in same cart
    private function addUniqueConstraintToCartIdInOrdersTable()
    {
        $sql = 'ALTER TABLE ' . _DB_PREFIX_ . 'orders ADD UNIQUE (id_cart)';
        Db::getInstance()->execute($sql);
    }

    private function registrationHook()
    {
        if (!$this->registerHook('paymentOptions')
            || !$this->registerHook('paymentReturn')
            || !$this->registerHook('displayProductPriceBlock')
            || !$this->registerHook('displayHeader')
            || !$this->registerHook('actionFrontControllerSetMedia')
        ) {
            return false;
        }

        return true;
    }

    public function uninstall()
    {
        $config = [
            'ESTO_SHOP_ID_EE',
            'ESTO_SECRET_KEY_EE',
            'ESTO_SHOP_ID_LV',
            'ESTO_SECRET_KEY_LV',
            'ESTO_SHOP_ID_LT',
            'ESTO_SECRET_KEY_LT',
            'ESTO_CONNECTION_MODE',
            'ESTO_SHOW_CHECKOUT_LOGO',
            'ESTO_MIN_ORDER_AMOUNT',
            'ESTO_MAX_ORDER_AMOUNT',
            'ESTO_MONTHLY_ENABLED_enable_estomonthly',
            'ESTO_MONTHLY_URL',
            'ESTO_MONTHLY_DESCRIPTION',
            'ESTO_CHECKOUT_LOGO',
            'ESTO_MONTHLY_LOGO',
            'ESTO_X_IS_ACTIVE',
            'ESTO_X_MIN_ORDER_AMOUNT',
            'ESTO_X_MAX_ORDER_AMOUNT',
            'ESTO_X_CHECKOUT_LOGO',
            'ESTO_PAY_LATER_IS_ACTIVE',
            'ESTO_PAY_LATER_MIN_ORDER_AMOUNT',
            'ESTO_PAY_LATER_MAX_ORDER_AMOUNT',
            'ESTO_PAY_LATER_CHECKOUT_LOGO',
            'ESTO_PAY_IS_ACTIVE',
            'ESTO_PAY_CHECKOUT_LOGO',
            'ESTO_USE_SPECIFIC_ENDPOINT_EE',
            'ESTO_USE_SPECIFIC_ENDPOINT_LV',
            'ESTO_USE_SPECIFIC_ENDPOINT_LT',
            'ESTO_PAY_CARD_IS_ACTIVE',
            'ESTO_PAY_CARD_CHECKOUT_LOGO',
        ];

        foreach ($config as $var) {
            Configuration::deleteByName($var);
        }

        $this->uninstallBanksTable();

        if (!parent::uninstall()) {
            return false;
        }

        return true;
    }

    public function getContent()
    {
        $output = '';

        if (Tools::isSubmit('submit' . $this->name)) {
            $output = $this->postProcess();
        }

        return $output . $this->displayForm();
    }

    private function postProcess()
    {
        $output = '';
        $values = [];
        $keys = [
            'ESTO_SHOP_ID_EE' => ['type' => 'str', 'html' => false],
            'ESTO_SECRET_KEY_EE' => ['type' => 'str', 'html' => false],
            'ESTO_SHOP_ID_LV' => ['type' => 'str', 'html' => false],
            'ESTO_SECRET_KEY_LV' => ['type' => 'str', 'html' => false],
            'ESTO_SHOP_ID_LT' => ['type' => 'str', 'html' => false],
            'ESTO_SECRET_KEY_LT' => ['type' => 'str', 'html' => false],
            'ESTO_CREATE_ORDER' => ['type' => 'str', 'html' => false],
            'ESTO_USE_ORDER_REF_AS_REFERENCE' => ['type' => 'bool', 'html' => false],
            'ESTO_COUNTRY' => ['type' => 'str', 'html' => false],
            'ESTO_SHOW_CHECKOUT_LOGO' => ['type' => 'bool', 'html' => false],
            'ESTO_CONNECTION_MODE' => ['type' => 'str', 'html' => false],
            'ESTO_MIN_ORDER_AMOUNT' => ['type' => 'float', 'html' => false],
            'ESTO_MAX_ORDER_AMOUNT' => ['type' => 'float', 'html' => false],
            'ESTO_MONTHLY_ENABLED_enable_estomonthly' => ['type' => 'int', 'html' => false],
            'ESTO_CHECKOUT_LOGO_DELETE_delete' => ['type' => 'bool', 'html' => false],
            'ESTO_X_IS_ACTIVE' => ['suffix' => 'active', 'type' => 'int', 'html' => false],
            'ESTO_X_CHECKOUT_CALC' => ['type' => 'bool', 'html' => false],
            'ESTO_X_MIN_ORDER_AMOUNT' => ['type' => 'float', 'html' => false],
            'ESTO_X_MAX_ORDER_AMOUNT' => ['type' => 'float', 'html' => false],
            'ESTO_PAY_LATER_IS_ACTIVE' => ['type' => 'int', 'html' => false],
            'ESTO_PAY_LATER_MIN_ORDER_AMOUNT' => ['type' => 'float', 'html' => false],
            'ESTO_PAY_LATER_MAX_ORDER_AMOUNT' => ['type' => 'float', 'html' => false],
            'ESTO_PAY_IS_ACTIVE' => ['suffix' => 'active', 'type' => 'int', 'html' => false],
            'ESTO_PAY_CARD_IS_ACTIVE' => ['suffix' => 'active', 'type' => 'int', 'html' => false],
            'ESTO_USE_SPECIFIC_ENDPOINT_EE' => ['type' => 'bool', 'html' => false],
            'ESTO_USE_SPECIFIC_ENDPOINT_LV' => ['type' => 'bool', 'html' => false],
            'ESTO_USE_SPECIFIC_ENDPOINT_LT' => ['type' => 'bool', 'html' => false],

            // Multilang fields
            'ESTO_DISPLAY_NAME' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_DISPLAY_DESCRIPTION' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_X_DISPLAY_NAME' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_X_DISPLAY_DESCRIPTION' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_PAY_LATER_DISPLAY_NAME' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_PAY_LATER_DISPLAY_DESCRIPTION' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_PAY_DISPLAY_NAME' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_PAY_DISPLAY_DESCRIPTION' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_PAY_CARD_DISPLAY_NAME' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_PAY_CARD_DISPLAY_DESCRIPTION' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_MONTHLY_URL' => ['lang' => true, 'type' => 'str', 'html' => false],
            'ESTO_MONTHLY_DESCRIPTION' => ['lang' => true, 'type' => 'str', 'html' => true],
        ];

        $activeLangs = Language::getLanguages(true);

        // Get data sent from $_POST
        foreach ($keys as $key => $info) {
            $functionName = $info['type'] . 'val'; // Either strval, floatval or intval

            if (!isset($info['lang']) || $info['lang'] === false) {
                $values[$key] = $functionName(Tools::getValue($key));
                continue;
            }

            if (!isset($values[$key])) {
                $values[$key] = [];
            }

            foreach ($activeLangs as $lang) {
                $values[$key][$lang['id_lang']] = $functionName(Tools::getValue($key . '_' . $lang['id_lang']));
            }
        }

        // Check if data was inserted correctly
        if(!$this->validateCredentials(Tools::getValue('ESTO_COUNTRY'))){
            return $this->displayError(
                $this->l('Please fill Shop Id and Secret key for selected primary endpoint country')
            );
        } elseif (!$this->validateSpecificCountryCredentials(
            Tools::getValue('ESTO_USE_SPECIFIC_ENDPOINT_EE'),
            Tools::getValue('ESTO_USE_SPECIFIC_ENDPOINT_LV'),
            Tools::getValue('ESTO_USE_SPECIFIC_ENDPOINT_LT')
        )) {
            return $this->displayError(
                $this->l('Please fill Shop Id and Secret key for all selected specific endpoint countries')
            );
        }
        if (/*!$values['ESTO_SHOP_ID'] ||
            !$values['ESTO_SECRET_KEY'] ||*/
            is_null($values['ESTO_CONNECTION_MODE']) ||
            !$values['ESTO_MIN_ORDER_AMOUNT'] ||
            !$values['ESTO_MAX_ORDER_AMOUNT']
        ) {
            return $this->displayError($this->l('Invalid Configuration value'));
        }

        // Update configuration
        foreach ($keys as $key => $info) {
            Configuration::updateValue($key, $values[$key], $info['html']);
        }

        // Save & delete logos
        foreach ($activeLangs as $lang) {
            $output .= $this->updateLogo('ESTO_CHECKOUT_LOGO', $lang['id_lang']);
            $output .= $this->updateLogo('ESTO_X_CHECKOUT_LOGO', $lang['id_lang']);
            $output .= $this->updateLogo('ESTO_PAY_LATER_CHECKOUT_LOGO', $lang['id_lang']);
            $output .= $this->updateLogo('ESTO_PAY_CHECKOUT_LOGO', $lang['id_lang']);
            $output .= $this->updateLogo('ESTO_PAY_CARD_CHECKOUT_LOGO', $lang['id_lang']);
            $output .= $this->updateLogo('ESTO_MONTHLY_LOGO', $lang['id_lang']);
        }

        $this->deleteLogo('ESTO_CHECKOUT_LOGO');
        $this->deleteLogo('ESTO_X_CHECKOUT_LOGO');
        $this->deleteLogo('ESTO_PAY_LATER_CHECKOUT_LOGO');
        $this->deleteLogo('ESTO_PAY_CHECKOUT_LOGO');
        $this->deleteLogo('ESTO_PAY_CARD_CHECKOUT_LOGO');
        $this->deleteLogo('ESTO_MONTHLY_LOGO');

        // Save enabled languages
        $this->updateEnabledLanguages();

        // clear cache for banks in supported country
        $sql = 'DELETE FROM `'._DB_PREFIX_.'esto_banks`';
        Db::getInstance()->Execute($sql);
        // clear cache for banks in supported country

        return $output . $this->displayConfirmation($this->l('Settings updated'));
    }

    private function createOrderStates()
    {
        foreach (['ESTO', 'ESTO_X', 'ESTO_PAY_LATER', 'ESTO_PAY'] as $estoPayment) {
            if ((bool) Configuration::get($estoPayment . '_OS_PROCESSING')) {
                continue;
            }

            $orderState = new OrderState();
            $orderState->name = [];

            foreach (Language::getLanguages() as $language) {
                if (Tools::strtolower($language['iso_code']) == 'et') {
                    $text = $estoPayment === 'ESTO_PAY' ? ' makse' : ' järelmaks';
                    $orderState->name[$language['id_lang']] = str_replace('_', ' ', $estoPayment) . $text . ' pooleli';
                } else {
                    $text = $estoPayment === 'ESTO_PAY' ? ' payment' : ' hire-purchase';
                    $orderState->name[$language['id_lang']] = str_replace('_', ' ', $estoPayment) . $text . ' processing';
                }
            }

            $orderState->send_email = false;
            $orderState->color = '#4169E1';
            $orderState->hidden = false;
            $orderState->delivery = false;
            $orderState->logable = false;
            $orderState->unremovable = true;
            $orderState->invoice = false;

            $orderState->add();
            Configuration::updateValue($estoPayment . '_OS_PROCESSING', (int) $orderState->id);
        }
    }

    public function displayForm()
    {
        // Get default language
        $defaultLang = (int) Configuration::get('PS_LANG_DEFAULT');
        $activeLangs = Language::getLanguages(true);

        // Init Fields form array
        $fieldsForm[0]['form'] = $this->getForm();

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;

        // Language
        $helper->languages = $this->context->controller->getlanguages();
        $helper->default_form_language = $defaultLang;
        $helper->allow_employee_form_lang = $defaultLang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = true;
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = [
            'save' => [
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                    '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list'),
            ],
        ];

        // Load current value
        $helper->fields_value['ESTO_SHOP_ID_EE'] = Configuration::get('ESTO_SHOP_ID_EE');
        $helper->fields_value['ESTO_SECRET_KEY_EE'] = Configuration::get('ESTO_SECRET_KEY_EE');
        $helper->fields_value['ESTO_SHOP_ID_LV'] = Configuration::get('ESTO_SHOP_ID_LV');
        $helper->fields_value['ESTO_SECRET_KEY_LV'] = Configuration::get('ESTO_SECRET_KEY_LV');
        $helper->fields_value['ESTO_SHOP_ID_LT'] = Configuration::get('ESTO_SHOP_ID_LT');
        $helper->fields_value['ESTO_SECRET_KEY_LT'] = Configuration::get('ESTO_SECRET_KEY_LT');
        $helper->fields_value['ESTO_CREATE_ORDER'] = Configuration::get('ESTO_CREATE_ORDER');
        $helper->fields_value['ESTO_USE_ORDER_REF_AS_REFERENCE'] = Configuration::get('ESTO_USE_ORDER_REF_AS_REFERENCE');
        $helper->fields_value['ESTO_COUNTRY'] = Configuration::get('ESTO_COUNTRY');
        $helper->fields_value['ESTO_SHOW_CHECKOUT_LOGO'] = Configuration::get('ESTO_SHOW_CHECKOUT_LOGO');
        $helper->fields_value['ESTO_CONNECTION_MODE'] = Configuration::get('ESTO_CONNECTION_MODE');
        $helper->fields_value['ESTO_MIN_ORDER_AMOUNT'] = Configuration::get('ESTO_MIN_ORDER_AMOUNT');
        $helper->fields_value['ESTO_MAX_ORDER_AMOUNT'] = Configuration::get('ESTO_MAX_ORDER_AMOUNT');
        $helper->fields_value['ESTO_MONTHLY_ENABLED_enable_estomonthly'] = Configuration::get('ESTO_MONTHLY_ENABLED_enable_estomonthly');
        $helper->fields_value['ESTO_CHECKOUT_LOGO_DELETE_delete'] = Configuration::get('ESTO_CHECKOUT_LOGO_DELETE_delete');
        $helper->fields_value['ESTO_X_IS_ACTIVE'] = Configuration::get('ESTO_X_IS_ACTIVE');
        $helper->fields_value['ESTO_X_MIN_ORDER_AMOUNT'] = Configuration::get('ESTO_X_MIN_ORDER_AMOUNT');
        $helper->fields_value['ESTO_X_MAX_ORDER_AMOUNT'] = Configuration::get('ESTO_X_MAX_ORDER_AMOUNT');
        $helper->fields_value['ESTO_X_CHECKOUT_CALC'] = Configuration::get('ESTO_X_CHECKOUT_CALC');
        $helper->fields_value['ESTO_PAY_LATER_IS_ACTIVE'] = Configuration::get('ESTO_PAY_LATER_IS_ACTIVE');
        $helper->fields_value['ESTO_PAY_LATER_MIN_ORDER_AMOUNT'] = Configuration::get('ESTO_PAY_LATER_MIN_ORDER_AMOUNT');
        $helper->fields_value['ESTO_PAY_LATER_MAX_ORDER_AMOUNT'] = Configuration::get('ESTO_PAY_LATER_MAX_ORDER_AMOUNT');
        $helper->fields_value['ESTO_PAY_IS_ACTIVE'] = Configuration::get('ESTO_PAY_IS_ACTIVE');
        $helper->fields_value['ESTO_PAY_CARD_IS_ACTIVE'] = Configuration::get('ESTO_PAY_CARD_IS_ACTIVE');
        $helper->fields_value['ESTO_X_CHECKOUT_LOGO_DELETE'] = false;
        $helper->fields_value['ESTO_PAY_LATER_CHECKOUT_LOGO_DELETE'] = false;
        $helper->fields_value['ESTO_PAY_CHECKOUT_LOGO_DELETE'] = false;
        $helper->fields_value['ESTO_PAY_CARD_CHECKOUT_LOGO_DELETE'] = false;
        $helper->fields_value['ESTO_USE_SPECIFIC_ENDPOINT_EE'] = Configuration::get('ESTO_USE_SPECIFIC_ENDPOINT_EE');
        $helper->fields_value['ESTO_USE_SPECIFIC_ENDPOINT_LV'] = Configuration::get('ESTO_USE_SPECIFIC_ENDPOINT_LV');
        $helper->fields_value['ESTO_USE_SPECIFIC_ENDPOINT_LT'] = Configuration::get('ESTO_USE_SPECIFIC_ENDPOINT_LT');

        foreach ($activeLangs as $lang) {
            $helper->fields_value['ESTO_MONTHLY_ENABLED_LANGS_id' . $lang['id_lang']] = Configuration::get('ESTO_MONTHLY_ENABLED_LANGS_id' . $lang['id_lang']);
            $helper->fields_value['ESTO_DISPLAY_NAME'][$lang['id_lang']] = Configuration::get('ESTO_DISPLAY_NAME', $lang['id_lang']);
            $helper->fields_value['ESTO_DISPLAY_DESCRIPTION'][$lang['id_lang']] = Configuration::get('ESTO_DISPLAY_DESCRIPTION', $lang['id_lang']);
            $helper->fields_value['ESTO_X_DISPLAY_NAME'][$lang['id_lang']] = Configuration::get('ESTO_X_DISPLAY_NAME', $lang['id_lang']);
            $helper->fields_value['ESTO_X_DISPLAY_DESCRIPTION'][$lang['id_lang']] = Configuration::get('ESTO_X_DISPLAY_DESCRIPTION', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_LATER_DISPLAY_NAME'][$lang['id_lang']] = Configuration::get('ESTO_PAY_LATER_DISPLAY_NAME', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_LATER_DISPLAY_DESCRIPTION'][$lang['id_lang']] = Configuration::get('ESTO_PAY_LATER_DISPLAY_DESCRIPTION', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_DISPLAY_NAME'][$lang['id_lang']] = Configuration::get('ESTO_PAY_DISPLAY_NAME', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_DISPLAY_DESCRIPTION'][$lang['id_lang']] = Configuration::get('ESTO_PAY_DISPLAY_DESCRIPTION', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_CARD_DISPLAY_NAME'][$lang['id_lang']] = Configuration::get('ESTO_PAY_CARD_DISPLAY_NAME', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_CARD_DISPLAY_DESCRIPTION'][$lang['id_lang']] = Configuration::get('ESTO_PAY_CARD_DISPLAY_DESCRIPTION', $lang['id_lang']);
            $helper->fields_value['ESTO_MONTHLY_URL'][$lang['id_lang']] = Configuration::get('ESTO_MONTHLY_URL', $lang['id_lang']);
            $helper->fields_value['ESTO_MONTHLY_DESCRIPTION'][$lang['id_lang']] = Configuration::get('ESTO_MONTHLY_DESCRIPTION', $lang['id_lang']);
            $helper->fields_value['ESTO_CHECKOUT_LOGO'][$lang['id_lang']] = Configuration::get('ESTO_CHECKOUT_LOGO', $lang['id_lang']);
            $helper->fields_value['ESTO_X_CHECKOUT_LOGO'][$lang['id_lang']] = Configuration::get('ESTO_X_CHECKOUT_LOGO', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_LATER_CHECKOUT_LOGO'][$lang['id_lang']] = Configuration::get('ESTO_PAY_LATER_CHECKOUT_LOGO', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_CHECKOUT_LOGO'][$lang['id_lang']] = Configuration::get('ESTO_PAY_CHECKOUT_LOGO', $lang['id_lang']);
            $helper->fields_value['ESTO_PAY_CARD_CHECKOUT_LOGO'][$lang['id_lang']] = Configuration::get('ESTO_PAY_CARD_CHECKOUT_LOGO', $lang['id_lang']);
            $helper->fields_value['ESTO_MONTHLY_LOGO'][$lang['id_lang']] = Configuration::get('ESTO_MONTHLY_LOGO', $lang['id_lang']);
        }

        return $helper->generateForm($fieldsForm);
    }

    private function getForm()
    {
        // Get default language
        $activeLangs = Language::getLanguages(true);
        $langSet = [];

        foreach ($activeLangs as $lang) {
            $langSet[] = [
                'id' => 'id' . $lang['id_lang'],
                'name' => $lang['name'],
                'val' => '1',
                'checked' => '',
            ];
        }

        $country_array = array(
            0 => array('esto_name'=>self::COUNTRY_ESTONIA,'name'=>$this->l(Tools::ucfirst(self::COUNTRY_ESTONIA))),
            1 => array('esto_name'=>self::COUNTRY_LITHUANIA,'name'=>$this->l(Tools::ucfirst(self::COUNTRY_LITHUANIA))),
            2 => array('esto_name'=>self::COUNTRY_LATVIA,'name'=>$this->l(Tools::ucfirst(self::COUNTRY_LATVIA))),
            3 => array('esto_name'=>self::COUNTRY_FINLAND,'name'=>$this->l(Tools::ucfirst(self::COUNTRY_FINLAND))),
        );

        $form = [
            'tabs' => [
                'general' => $this->l('General settings'),
                'esto' => $this->l('ESTO'),
                'estoX' => $this->l('ESTO X'),
                'estoPayLater' => $this->l('ESTO Pay Later'),
                'estoPay' => $this->l('ESTO Pay'),
                'estoPayCard' => $this->l('Card payments (ESTO Pay)'),
            ],
            'tinymce' => true,
            'legend' => [
                'title' => $this->l('MODULE SETTINGS'),
                'icon' => 'icon-cogs',
            ],
            'input' => [
                [
                    'tab' => 'general',
                    'type' => 'select',
                    'label' => $this->l('Connection mode'),
                    'hint' => $this->l('Use test mode for development purposes only'),
                    'name' => 'ESTO_CONNECTION_MODE',
                    'required' => 'true',
                    'options' => [
                        'query' => [
                            [
                                'id' => self::MODE_TEST,
                                'name' => $this->l('Test'),
                            ],
                            [
                                'id' => self::MODE_LIVE,
                                'name' => $this->l('Live'),
                            ],
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
                [
                    'tab' => 'general',
                    'type' => 'text',
                    'label' => $this->l('Estonian endpoint Shop ID'),
                    'desc' => $this->l('Shop ID is set by ESTO and can be retrieved from') . ' <a href="https://partner.esto.ee/settings" target="_blank">https://partner.esto.ee/settings</a>',
                    'name' => 'ESTO_SHOP_ID_EE',
                    'size' => 10,
                    'required' => false,
                ],
                [
                    'tab' => 'general',
                    'type' => 'text',
                    'label' => $this->l('Estonian endpoint Secret key'),
                    'desc' => $this->l('Secret API key should be kept secret and it can be obtained from') . ' <a href="https://partner.esto.ee/settings" target="_blank">https://partner.esto.ee/settings</a>',
                    'name' => 'ESTO_SECRET_KEY_EE',
                    'size' => 10,
                    'required' => false,
                ],
                [
                    'tab' => 'general',
                    'type' => 'text',
                    'label' => $this->l('Latvian endpoint Shop ID'),
                    'desc' => $this->l('Shop ID is set by ESTO and can be retrieved from') . ' <a href="https://partner.esto.ee/settings" target="_blank">https://partner.esto.ee/settings</a>',
                    'name' => 'ESTO_SHOP_ID_LV',
                    'size' => 10,
                    'required' => false,
                ],
                [
                    'tab' => 'general',
                    'type' => 'text',
                    'label' => $this->l('Latvian endpoint Secret key'),
                    'desc' => $this->l('Secret API key should be kept secret and it can be obtained from') . ' <a href="https://partner.esto.ee/settings" target="_blank">https://partner.esto.ee/settings</a>',
                    'name' => 'ESTO_SECRET_KEY_LV',
                    'size' => 10,
                    'required' => false,
                ],
                [
                    'tab' => 'general',
                    'type' => 'text',
                    'label' => $this->l('Lithuanian endpoint Shop ID'),
                    'desc' => $this->l('Shop ID is set by ESTO and can be retrieved from') . ' <a href="https://partner.esto.ee/settings" target="_blank">https://partner.esto.ee/settings</a>',
                    'name' => 'ESTO_SHOP_ID_LT',
                    'size' => 10,
                    'required' => false,
                ],
                [
                    'tab' => 'general',
                    'type' => 'text',
                    'label' => $this->l('Lithuanian endpoint Secret key'),
                    'desc' => $this->l('Secret API key should be kept secret and it can be obtained from') . ' <a href="https://partner.esto.ee/settings" target="_blank">https://partner.esto.ee/settings</a>',
                    'name' => 'ESTO_SECRET_KEY_LT',
                    'size' => 10,
                    'required' => false,
                ],
                [
                    'tab' => 'general',
                    'type' => 'select',
                    'label' => $this->l('Create order'),
                    'name' => 'ESTO_CREATE_ORDER',
                    'required' => true,
                    'options' => [
                        'query' => [
                            [
                                'id' => self::CREATE_ORDER_BEFORE,
                                'name' => $this->l('Before signing'),
                            ],
                            [
                                'id' => self::CREATE_ORDER_AFTER,
                                'name' => $this->l('After signing'),
                            ],
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
                [
                    'tab' => 'general',
                    'type' => 'select',
                    'label' => $this->l('Order identifier'),
                    'desc' => $this->l('Used only when creating order before payment'),
                    'name' => 'ESTO_USE_ORDER_REF_AS_REFERENCE',
                    'options' => [
                        'query' => [
                            [
                                'id' => false,
                                'name' => $this->l('Order ID'),
                            ],
                            [
                                'id' => true,
                                'name' => $this->l('Order reference'),
                            ],
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
                [
                    'tab' => 'general',
                    'type' => 'select',
                    'label' => $this->l('Primary endpoint'),
                    'desc' => $this->l('Select correct endpoint for your country'),
                    'name' => 'ESTO_COUNTRY',
                    'required' => true,
                    'options' => array(
                        'query' => $country_array,
                        'id' => 'esto_name',
                        'name' => 'name',
                    ),
                ],
                [
                    'tab' => 'general',
                    'type' => 'checkbox',
                    'label' => $this->l('Use specific endpoint?'),
                    'name' => 'ESTO_USE_SPECIFIC_ENDPOINT',
                    'values' => [
                        'query' => [
                            [
                                'id' => 'EE', # this gets appended to main option name using '_'
                                'name' => $this->l('for Estonia'),
                                'val' => '1',
                                'checked' => '',
                            ],
                            [
                                'id' => 'LV',
                                'name' => $this->l('for Latvia'),
                                'val' => '1',
                                'checked' => '',
                            ],
                            [
                                'id' => 'LT',
                                'name' => $this->l('for Lithuania'),
                                'val' => '1',
                                'checked' => '',
                            ],
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
                [
                    'tab' => 'general',
                    'type' => 'switch',
                    'label' => $this->l('Show ESTO logos at checkout'),
                    'name' => 'ESTO_SHOW_CHECKOUT_LOGO',
                    'values' => [
                        [
                            'id' => 'active_on',
                            'value' => true,
                            'checked' => $this->l('Enabled'),
                        ],
                        [
                            'id' => 'active_off',
                            'value' => false,
                            'checked' => $this->l('Disabled'),
                        ],
                    ],
                ],
                [
                    'tab' => 'esto',
                    'type' => 'checkbox',
                    'label' => $this->l('Enabled in languages'),
                    'name' => 'ESTO_MONTHLY_ENABLED_LANGS',
                    'values' => [
                        'query' => $langSet,
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
                [
                    'tab' => 'esto',
                    'type' => 'text',
                    'label' => $this->l('Esto lease name'),
                    'name' => 'ESTO_DISPLAY_NAME',
                    'lang' => true,
                ],
                [
                    'tab' => 'esto',
                    'type' => 'text',
                    'label' => $this->l('Esto lease description'),
                    'name' => 'ESTO_DISPLAY_DESCRIPTION',
                    'lang' => true,
                ],
                [
                    'tab' => 'esto',
                    'type' => 'text',
                    'label' => $this->l('Min. order amount'),
                    'hint' => $this->l('The minimum order amount required for displaying the module in checkout'),
                    'name' => 'ESTO_MIN_ORDER_AMOUNT',
                    'size' => 10,
                    'required' => true,
                ],
                [
                    'tab' => 'esto',
                    'type' => 'text',
                    'label' => $this->l('Max. order amount'),
                    'hint' => $this->l('The maximum order amount allowed for displaying the module in checkout'),
                    'name' => 'ESTO_MAX_ORDER_AMOUNT',
                    'size' => 10,
                    'required' => true,
                ],
                [
                    'tab' => 'esto',
                    'type' => 'file_lang',
                    'label' => $this->l('Checkout logo'),
                    'name' => 'ESTO_CHECKOUT_LOGO',
                    'lang' => true,
                ],
                [
                    'tab' => 'esto',
                    'type' => 'checkbox',
                    'label' => $this->l('Delete checkout logo?'),
                    'name' => 'ESTO_CHECKOUT_LOGO_DELETE',
                    'values' => [
                        'query' => [
                            [
                                'id' => 'delete',
                                'name' => '',
                                'val' => '1',
                                'checked' => '',
                            ],
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
                [
                    'tab' => 'esto',
                    'type' => 'checkbox',
                    'label' => $this->l('Enable monthly payment price on product page'),
                    'name' => 'ESTO_MONTHLY_ENABLED',
                    'values' => [
                        'query' => [
                            [
                                'id' => 'enable_estomonthly',
                                'name' => '',
                                'val' => '1',
                                'checked' => '',
                            ],
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
                [
                    'tab' => 'esto',
                    'type' => 'text',
                    'label' => $this->l('Monthly payment info URL'),
                    'desc' => $this->l('Must be a full URL if used (e.g. "https://www.esto.ee")'),
                    'name' => 'ESTO_MONTHLY_URL',
                    'lang' => true,
                ],
                [
                    'tab' => 'esto',
                    'type' => 'textarea',
                    'label' => $this->l('Custom text under monthly payment'),
                    'name' => 'ESTO_MONTHLY_DESCRIPTION',
                    'autoload_rte' => true,
                    'lang' => true,
                ],
                [
                    'tab' => 'esto',
                    'type' => 'file_lang',
                    'label' => $this->l('Logo'),
                    'hint' => $this->l('Used for monthly payment and on checkout page'),
                    'name' => 'ESTO_MONTHLY_LOGO',
                    'lang' => true,
                ],
                [
                    'tab' => 'esto',
                    'type' => 'checkbox',
                    'label' => $this->l('Delete monthly logo?'),
                    'name' => 'ESTO_MONTHLY_LOGO_DELETE',
                    'values' => [
                        'query' => [
                            [
                                'id' => 'delete',
                                'name' => '',
                                'val' => '1',
                                'checked' => '',
                            ],
                        ],
                        'id' => 'id',
                        'name' => 'name',
                    ],
                ],
            ],
            'submit' => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ],
        ];

        $formEstoX = [
            [
                'tab' => 'estoX',
                'type' => 'switch',
                'label' => $this->l('Enable ESTO X payment option'),
                'name' => 'ESTO_X_IS_ACTIVE',
                'hint' => $this->l('ESTO X is an alternative ESTO payment method. For more information and activation please contact ESTO Partner Support.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => 'estoX',
                'type' => 'text',
                'label' => $this->l('Esto X name'),
                'name' => 'ESTO_X_DISPLAY_NAME',
                'lang' => true,
            ],
            [
                'tab' => 'estoX',
                'type' => 'text',
                'label' => $this->l('Esto X description'),
                'name' => 'ESTO_X_DISPLAY_DESCRIPTION',
                'lang' => true,
            ],
            [
                'tab' => 'estoX',
                'type' => 'text',
                'label' => $this->l('Min. order amount'),
                'hint' => $this->l('The minimum order amount required for displaying the module in checkout'),
                'name' => 'ESTO_X_MIN_ORDER_AMOUNT',
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => 'estoX',
                'type' => 'text',
                'label' => $this->l('Max. order amount'),
                'hint' => $this->l('The maximum order amount allowed for displaying the module in checkout'),
                'name' => 'ESTO_X_MAX_ORDER_AMOUNT',
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => 'estoX',
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => 'ESTO_X_CHECKOUT_LOGO',
                'lang' => true,
            ],
            [
                'tab' => 'estoX',
                'type' => 'switch',
                'label' => $this->l('Delete checkout logo?'),
                'name' => 'ESTO_X_CHECKOUT_LOGO_DELETE',
                'values' => [
                    [
                        'id' => 'delete_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'delete_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => 'estoX',
                'type' => 'switch',
                'label' => $this->l('Enable checkout calculator'),
                'name' => 'ESTO_X_CHECKOUT_CALC',
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];

        $formEstoPayLater = [
            [
                'tab' => 'estoPayLater',
                'type' => 'switch',
                'label' => $this->l('Enable ESTO Pay Later payment option'),
                'name' => 'ESTO_PAY_LATER_IS_ACTIVE',
                'hint' => $this->l('Pay Later is an alternative ESTO payment method. For more information and activation please contact ESTO Partner Support.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => 'estoPayLater',
                'type' => 'text',
                'label' => $this->l('Esto pay later name'),
                'name' => 'ESTO_PAY_LATER_DISPLAY_NAME',
                'lang' => true,
            ],
            [
                'tab' => 'estoPayLater',
                'type' => 'text',
                'label' => $this->l('Esto pay later description'),
                'name' => 'ESTO_PAY_LATER_DISPLAY_DESCRIPTION',
                'lang' => true,
            ],
            [
                'tab' => 'estoPayLater',
                'type' => 'text',
                'label' => $this->l('Min. order amount'),
                'hint' => $this->l('The minimum order amount required for displaying the module in checkout'),
                'name' => 'ESTO_PAY_LATER_MIN_ORDER_AMOUNT',
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => 'estoPayLater',
                'type' => 'text',
                'label' => $this->l('Max. order amount'),
                'hint' => $this->l('The maximum order amount allowed for displaying the module in checkout'),
                'name' => 'ESTO_PAY_LATER_MAX_ORDER_AMOUNT',
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => 'estoPayLater',
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => 'ESTO_PAY_LATER_CHECKOUT_LOGO',
                'lang' => true,
            ],
            [
                'tab' => 'estoPayLater',
                'type' => 'switch',
                'label' => $this->l('Delete checkout logo?'),
                'name' => 'ESTO_PAY_LATER_CHECKOUT_LOGO_DELETE',
                'values' => [
                    [
                        'id' => 'delete_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'delete_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];

        $formEstoPay = [
            [
                'tab' => 'estoPay',
                'type' => 'switch',
                'label' => $this->l('Enable ESTO Pay payment option'),
                'name' => 'ESTO_PAY_IS_ACTIVE',
                'hint' => $this->l('ESTO Pay bank payments are a direct payment method for all Baltic and Finnish banklinks. Contact ESTO Partner Support for additional information and activation.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => 'estoPay',
                'type' => 'text',
                'label' => $this->l('Esto pay name'),
                'name' => 'ESTO_PAY_DISPLAY_NAME',
                'lang' => true,
            ],
            [
                'tab' => 'estoPay',
                'type' => 'text',
                'label' => $this->l('Esto pay description'),
                'name' => 'ESTO_PAY_DISPLAY_DESCRIPTION',
                'lang' => true,
            ],
            [
                'tab' => 'estoPay',
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => 'ESTO_PAY_CHECKOUT_LOGO',
                'lang' => true,
            ],
            [
                'tab' => 'estoPay',
                'type' => 'switch',
                'label' => $this->l('Delete checkout logo?'),
                'name' => 'ESTO_PAY_CHECKOUT_LOGO_DELETE',
                'values' => [
                    [
                        'id' => 'delete_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'delete_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];

        $formEstoPayCard = [
            [
                'tab' => 'estoPayCard',
                'type' => 'switch',
                'label' => $this->l('Enable Card payments (ESTO Pay) payment option'),
                'name' => 'ESTO_PAY_CARD_IS_ACTIVE',
                'hint' => $this->l('ESTO Pay card payments are Visa and Mastercard credit/debit card payments. Contact ESTO Partner Support for additional information and activation.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => 'estoPayCard',
                'type' => 'text',
                'label' => $this->l('Card payments (ESTO Pay) name'),
                'name' => 'ESTO_PAY_CARD_DISPLAY_NAME',
                'lang' => true,
            ],
            [
                'tab' => 'estoPayCard',
                'type' => 'text',
                'label' => $this->l('Card payments (ESTO Pay) description'),
                'name' => 'ESTO_PAY_CARD_DISPLAY_DESCRIPTION',
                'lang' => true,
            ],
            [
                'tab' => 'estoPayCard',
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => 'ESTO_PAY_CARD_CHECKOUT_LOGO',
                'lang' => true,
            ],
            [
                'tab' => 'estoPayCard',
                'type' => 'switch',
                'label' => $this->l('Delete checkout logo?'),
                'name' => 'ESTO_PAY_CARD_CHECKOUT_LOGO_DELETE',
                'values' => [
                    [
                        'id' => 'delete_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'delete_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];

        $form['input'] = array_merge($form['input'], $formEstoX, $formEstoPayLater, $formEstoPay, $formEstoPayCard);

        return $form;
    }

    public function hookDisplayHeader($params)
    {
        $this->context->controller->registerStylesheet('modules-esto', 'modules/' . $this->name . '/views/css/checkout.css', ['media' => 'all', 'priority' => 150]);
    }

    public function hookPaymentOptions($params)
    {

        $cart = isset($params['cart']) ? $params['cart'] : $this->context->cart;
        $paymentOptions = [];

        $configurations = Configuration::getMultiple([
            'ESTO_PAY_DISPLAY_NAME',
            'ESTO_PAY_DISPLAY_DESCRIPTION',
            'ESTO_DISPLAY_NAME',
            'ESTO_DISPLAY_DESCRIPTION',
            'ESTO_X_DISPLAY_NAME',
            'ESTO_X_DISPLAY_DESCRIPTION',
            'ESTO_PAY_LATER_DISPLAY_NAME',
            'ESTO_PAY_LATER_DISPLAY_DESCRIPTION',
            'ESTO_PAY_CARD_DISPLAY_NAME',
            'ESTO_PAY_CARD_DISPLAY_DESCRIPTION',
        ], $this->context->language->id);


        $sql = 'select * FROM `'._DB_PREFIX_.'esto_banks`';
        $result = Db::getInstance()->getRow($sql);


        $date_update = isset($result['date_update'])?$result['date_update']:0;
        $id_item = isset($result['id'])?$result['id']:null;
        if ((strtotime(date("Y-m-d H:i:s")) - strtotime($date_update)) > self::CACHE_TIME * 60) {

            $api_details = self::getDefaultCountryApiData();
            $api = new EstoAPI($api_details['url'], $api_details['user'], $api_details['pass'], $api_details['country_iso']);
            $data_banks = $api->getAvailablePayments();

            if($id_item){
                $sql = 'DELETE FROM `'._DB_PREFIX_.'esto_banks` WHERE id =  '.(int)$id_item.'';
                Db::getInstance()->Execute($sql);
            }

            $sql = 'INSERT INTO `'._DB_PREFIX_.'esto_banks` SET `data` = \''.pSQL(serialize($data_banks)).'\', date_update = \''.pSQL(date("Y-m-d H:i:s")).'\'';
            Db::getInstance()->Execute($sql);

        } else {
            $data_banks = isset($result['data'])?unserialize($result['data']):array();
        }
        $default_country = self::API_COUNTRIES[Configuration::get('ESTO_COUNTRY')];
        if (!empty($params['cart']->id_address_invoice)) {
            $billing_address = new Address($params['cart']->id_address_invoice);
            $api_data = $this->getApiDataByAddress($billing_address);
            if(!empty($api_data)) {
                $default_country = $api_data['country_iso'];
            }
        }

        $card_option = $this->getFirstCardOption($data_banks, $default_country);

        $this->context->smarty->assign([
            $this->name.'countries_for_banks' => self::API_COUNTRIES,
            $this->name.'default_country' => $default_country,
            $this->name.'data_banks' => $data_banks,
            $this->name.'esto_payment' => 'ESTO_PAY',
            $this->name.'_card_type_name' => self::CARD_TYPE_NAME,
        ]);

        $html_banks = $this->display(__FILE__, 'views/templates/hook/banks.tpl');

        $esto_x_calc_html = '';
        $showEstoXcheckout = Configuration::get('ESTO_X_CHECKOUT_CALC');
        if($showEstoXcheckout && Configuration::get('ESTO_X_IS_ACTIVE')) {
            $image_path =  $this->context->shop->getBaseURL(true) . "/modules/esto/views/img/";
            $total = $cart->getOrderTotal();
            $smartyEstoXVars['esto_x_payments'] = $this->getEstoXPayments($total);
            $smartyEstoXVars['image_path'] = $image_path;
            $this->context->smarty->assign($smartyEstoXVars);
            $esto_x_calc_html = $this->display(__FILE__, 'views/templates/hook/estoxcalc.tpl');
        }

        $estoPayments = [
            'ESTO_PAY' => [
                'callToActionText' => !empty($configurations['ESTO_PAY_DISPLAY_NAME'])
                    ? $configurations['ESTO_PAY_DISPLAY_NAME']
                    : $this->l('Pay in the bank'),
                'additionalInformation' => (
                    !empty($configurations['ESTO_PAY_DISPLAY_DESCRIPTION'])
                        ? $configurations['ESTO_PAY_DISPLAY_DESCRIPTION']
                        : $this->l('Payment is made using a secure payment solution called KEVIN (UAB “KEVIN EU”), which is licensed by the Bank of Lithuania.')
                    ).$html_banks,
            ],
            'ESTO' => [
                'callToActionText' => !empty($configurations['ESTO_DISPLAY_NAME'])
                    ? $configurations['ESTO_DISPLAY_NAME']
                    : $this->l('ESTO hire purchase'),
                'additionalInformation' =>
                    !empty($configurations['ESTO_DISPLAY_DESCRIPTION'])
                        ? $configurations['ESTO_DISPLAY_DESCRIPTION']
                        : $this->l('Confirm a purchase in 60 seconds and pay in installments, conveniently and quickly. Just pay later!'),
            ],
            'ESTO_X' => [
                'callToActionText' => !empty($configurations['ESTO_X_DISPLAY_NAME'])
                    ? $configurations['ESTO_X_DISPLAY_NAME']
                    : $this->l('Pay 3 in equal parts. At no extra charge.'),
                'additionalInformation' =>
                    !empty($configurations['ESTO_X_DISPLAY_DESCRIPTION'])
                        ? $configurations['ESTO_X_DISPLAY_DESCRIPTION']
                        : $this->l('ESTO 3 - pay without additional cost within 3 months! Just pay later.').$esto_x_calc_html,
            ],
            'ESTO_PAY_LATER' => [
                'callToActionText' =>
                    !empty($configurations['ESTO_PAY_LATER_DISPLAY_NAME'])
                        ? $configurations['ESTO_PAY_LATER_DISPLAY_NAME']
                        : $this->l('Buy now and pay later!'),
                'additionalInformation' =>
                    !empty($configurations['ESTO_PAY_LATER_DISPLAY_DESCRIPTION'])
                        ? $configurations['ESTO_PAY_LATER_DISPLAY_DESCRIPTION']
                        : $this->l('Just confirm your order with 3 clicks and take your time! Pay later, within 30 days, without any extra fees!'),
            ],
        ];

        if (!empty($card_option)) {
            $estoPayments['ESTO_PAY_CARD'] = [
                'callToActionText' => !empty($configurations['ESTO_PAY_CARD_DISPLAY_NAME'])
                    ? $configurations['ESTO_PAY_CARD_DISPLAY_NAME']
                    : $this->l('Pay by card (Visa/Mastercard)'),
                'additionalInformation' =>
                    !empty($configurations['ESTO_PAY_CARD_DISPLAY_DESCRIPTION'])
                        ? $configurations['ESTO_PAY_CARD_DISPLAY_DESCRIPTION']
                        : $this->l('Payment is made using a secure payment solution.'),
                'actionLink' => $this->context->link->getModuleLink(
                    $this->name, 'payment', [
                        'payment-option' => 'ESTO_PAY',
                        'esto-type' => $card_option['type'],
                        'esto-bank' => $card_option['key'],
                        'esto-bank-country-code' => $card_option['country_code']
                    ]
                ),
            ];
        }

        foreach ($estoPayments as $estoPayment => $estoPaymentData) {
            if (!$this->validatePaymentOption($estoPayment, $cart)) {
                continue;
            }

            $paymentOptions[] = $this->getPaymentOption($estoPayment, $estoPaymentData);
        }

        return $paymentOptions;
    }

    private function getPaymentOption($payment, $estoPaymentData)
    {
        // Logo
        $lang_iso = $this->context->language->iso_code;
        $ds = DIRECTORY_SEPARATOR;
        $path = _PS_MODULE_DIR_.$this->name.$ds.'views'.$ds.'img'.$ds;
        $logoFileName = Configuration::get($payment . '_CHECKOUT_LOGO', $this->context->language->id);

        if (!$logoFileName) {
            if ($payment === 'ESTO_PAY') {
                $logoFileName = 'esto_pay_logo_en.svg';
            } elseif ($payment === 'ESTO_PAY_CARD') {
                $logoFileName = 'esto_pay_card_logo_en.svg';
            } elseif ($payment === 'ESTO_PAY_LATER') {
                $fileName = 'esto_pay_later_logo_'.$lang_iso.'.svg';
                $pathToLogo = $path . $fileName;
                $logoFileName = (file_exists($pathToLogo) ? $fileName : 'esto_pay_later_logo_en.svg');
            } elseif ($payment === 'ESTO_X') {
                $fileName = 'esto_x_logo_'.$lang_iso.'.svg';
                $pathToLogo = $path . $fileName;
                $logoFileName = (file_exists($pathToLogo) ? $fileName : 'esto_x_logo_en.svg');
            } elseif ($payment === 'ESTO') {
                $fileName = 'esto_logo_'.$lang_iso.'.svg';
                $pathToLogo = $path . $fileName;
                $logoFileName = (file_exists($pathToLogo) ? $fileName : 'esto_logo_en.svg');
            } else {
                $logoFileName = 'esto_logo_en.svg';
            }
        }

        $pathToLogo = $path . $logoFileName;

        $actionLink = isset($estoPaymentData['actionLink'])
            ? $estoPaymentData['actionLink']
            : $this->context->link->getModuleLink($this->name, 'payment', ['payment-option' => $payment]);

        $newOption = new PaymentOption();
        $newOption->setModuleName($this->name)
            ->setAction($actionLink);

        // Name displayed on the payment
        if (isset($estoPaymentData['callToActionText'])) {
            $newOption->setCallToActionText($estoPaymentData['callToActionText']);
        }

        // Additional information
        if (isset($estoPaymentData['additionalInformation'])) {
            $newOption->setAdditionalInformation($estoPaymentData['additionalInformation']);
        }

        if(Configuration::get('ESTO_SHOW_CHECKOUT_LOGO', true)) {
            $newOption->setLogo(Media::getMediaPath($pathToLogo));
        }

        return $newOption;
    }

    private function validatePaymentOption($estoPayment, $cart)
    {
        // @TODO correct configurations getting and setting with language
        if ('ESTO' === $estoPayment && !Configuration::get('ESTO_MONTHLY_ENABLED_LANGS_id' . $this->context->language->id)) {
            return false;
        }
        if ('ESTO_X' === $estoPayment && !Configuration::get('ESTO_X_IS_ACTIVE')) {
            return false;
        }
        if ('ESTO_PAY_LATER' === $estoPayment && !Configuration::get('ESTO_PAY_LATER_IS_ACTIVE')) {
            return false;
        }
        if ('ESTO_PAY' === $estoPayment && !Configuration::get('ESTO_PAY_IS_ACTIVE')) {
            return false;
        }

        if ('ESTO_PAY_CARD' === $estoPayment && !Configuration::get('ESTO_PAY_CARD_IS_ACTIVE')) {
            return false;
        }

        $api_details = self::getDefaultCountryApiData();
        if (empty($api_details)
            || !$api_details['url']
            || !$api_details['user']
            || !$api_details['pass']
            || !$api_details['country_iso']
            || !Configuration::get('ESTO_CONNECTION_MODE')
        ) {
            return false;
        }

        // Order total must be within the allowed range
        $orderTotal = $cart->getOrderTotal();
        if ('ESTO' === $estoPayment &&
            ($orderTotal > Configuration::get('ESTO_MAX_ORDER_AMOUNT') || $orderTotal < Configuration::get('ESTO_MIN_ORDER_AMOUNT'))
        ) {
            return false;
        }
        // The minimum allowed amount for ESTO X is 60€!
        if ('ESTO_X' === $estoPayment &&
            ($orderTotal > Configuration::get('ESTO_X_MAX_ORDER_AMOUNT') || $orderTotal < Configuration::get('ESTO_X_MIN_ORDER_AMOUNT'))
        ) {
            return false;
        }
        if ('ESTO_PAY_LATER' === $estoPayment &&
            ($orderTotal > Configuration::get('ESTO_PAY_LATER_MAX_ORDER_AMOUNT') || $orderTotal < Configuration::get('ESTO_PAY_LATER_MIN_ORDER_AMOUNT'))
        ) {
            return false;
        }

        return true;
    }

    public function hookPaymentReturn()
    {
    }

    public function hookDisplayProductPriceBlock($params)
    {
        if ('price' !== $params['type'] || !$this->isProductPageBlockAllowed($params)) {
            return;
        }

        $api_details = self::getDefaultCountryApiData();
        $api = new EstoAPI($api_details['url'], $api_details['user'], $api_details['pass'], $api_details['country_iso']);
        $response = $api->getMonthlyPayment(['amount' => $params['product']['price_amount']]);

        if (!$response) {
            return false;
        }

        $monthlyLogoFileName = Configuration::get('ESTO_MONTHLY_LOGO', $this->context->language->id);
        if (!$monthlyLogoFileName){
            $lang_iso = $this->context->language->iso_code;
            $ds = DIRECTORY_SEPARATOR;
            $monthlyLogoFileName = 'esto_logo_'.$lang_iso.'.svg';
            $path = _PS_MODULE_DIR_.$this->name.$ds.'views'.$ds.'img'.$ds;
            $pathToLogo = $path . $monthlyLogoFileName;
            $monthlyLogoFileName = (file_exists($pathToLogo) ? $monthlyLogoFileName : 'esto_logo_en.svg');
        }

        $this->context->smarty->assign([
            'monthly_payment' => $response,
            'url' => Configuration::get('ESTO_MONTHLY_URL', $this->context->language->id),
            'img' => $monthlyLogoFileName,
            'baseurl' => $this->context->shop->getBaseURL(true),
            'desc' => Configuration::get('ESTO_MONTHLY_DESCRIPTION', $this->context->language->id),
        ]);

        return $this->fetch('module:' . $this->name . '/views/templates/hook/estomonthly.tpl');
    }

    private function isProductPageBlockAllowed($params)
    {
        $values = Configuration::getMultiple([
            'ESTO_MONTHLY_ENABLED_enable_estomonthly',
            'ESTO_MONTHLY_ENABLED_LANGS_id' . $this->context->language->id,
            'ESTO_MIN_ORDER_AMOUNT',
            'ESTO_MAX_ORDER_AMOUNT',
        ]);

        $estoMin = floatval($values['ESTO_MIN_ORDER_AMOUNT']);
        $estoMax = floatval($values['ESTO_MAX_ORDER_AMOUNT']);

        if (!$values['ESTO_MONTHLY_ENABLED_enable_estomonthly'] ||
            !$values['ESTO_MONTHLY_ENABLED_LANGS_id' . $this->context->language->id] ||
            !isset($params['product']['id']) ||
            max(self::MIN_AMOUNT, $estoMin) > $params['product']['price_amount'] ||
            min(self::MAX_AMOUNT, $estoMax) < $params['product']['price_amount']
        ) {
            return false;
        }

        return true;
    }

    /**
     * @param string $logoName
     *
     * @return string
     */
    public function updateLogo($logoName, $idLang = 0)
    {
        $output = '';

        $logoKeyName = $logoName;
        $logoName = $logoName . '_' . $idLang;
        // Initial check - Is something uploaded at all
        if (!isset($_FILES[$logoName]) || !isset($_FILES[$logoName]['tmp_name']) || empty($_FILES[$logoName]['tmp_name'])) {
            return $output;
        }

        // Upload validation
        $imageValidationError = ImageManager::validateUpload($_FILES[$logoName]);
        if (false !== $imageValidationError) {
            return $output . $this->displayError($this->l('Image validation error') . ' - ' . $logoName);
        }

        // Uploading to a temporary destination
        $oldTmpName = $_FILES[$logoName]['tmp_name'];
        $newTmpName = tempnam(_PS_TMP_IMG_DIR_, 'PS');
        if (!$newTmpName || !move_uploaded_file($oldTmpName, $newTmpName)) {
            return $output . $this->displayConfirmation($this->l('Image saving error') . ' - ' . $logoName);
        }

        // Resizing the image uploaded to tmp destination
        // Saving to final destination after resizing
        $destinationFile = dirname(__FILE__) . '/views/img/' . $_FILES[$logoName]['name'];
        $isResizeSuccessful = ImageManager::resize($newTmpName, $destinationFile);
        if (!$isResizeSuccessful) {
            $output .= $this->displayConfirmation($this->l('Image resize error') . $logoName);

        // If image is uploaded successfully - update the filename in configuration
        } else {
            Configuration::updateValue($logoKeyName, [$idLang => $_FILES[$logoName]['name']]);
        }

        return $output;
    }

    /**
     * @param string $logoName
     */
    public function deleteLogo($logoName)
    {
        if (intval(Tools::getValue($logoName . '_DELETE_delete')) ||
            intval(Tools::getValue($logoName . '_DELETE'))
        ) {
            Configuration::updateValue($logoName, '');
            $multiLangData = [];
            foreach (Language::getLanguages() as $language) {
                if (Configuration::hasKey($logoName, $language['id_lang'])) {
                    $multiLangData[$language['id_lang']] = '';
                }
            }
            Configuration::updateValue($logoName, $multiLangData);
        }
    }

    public function updateEnabledLanguages()
    {
        foreach (Language::getLanguages(true) as $lang) {
            Configuration::updateValue(
                'ESTO_MONTHLY_ENABLED_LANGS_id' . $lang['id_lang'],
                Tools::getValue('ESTO_MONTHLY_ENABLED_LANGS_id' . $lang['id_lang'])
            );
        }
    }

    public static function getCountryApiData($country)
    {
        if (empty(self::API_COUNTRIES[$country])) {
            return false;
        }
        $country_iso = self::API_COUNTRIES[$country];
        if (empty(self::API_URLS[$country])) {
            return false;
        }
        $countryUrl = self::API_URLS[$country];
        $esto_shop_id = Configuration::get('ESTO_SHOP_ID_'.$country_iso);
        $esto_secret_key = Configuration::get('ESTO_SECRET_KEY_'.$country_iso);
        if (empty($esto_shop_id) || empty($esto_secret_key)) {
            return false;
        }
        return array(
            'url' => $countryUrl,
            'user' => $esto_shop_id,
            'pass' => $esto_secret_key,
            'country_iso' => $country_iso,
        );
    }

    public static function getDefaultCountryApiData()
    {
        $country = Configuration::get('ESTO_COUNTRY');
        if (array_key_exists($country, self::SUBSTITUTED_COUNTRIES)) {
            $country = self::SUBSTITUTED_COUNTRIES[$country];
        }
        return self::getCountryApiData($country);
    }

    public static function getApiDataByAddress($address)
    {
        if (empty($address)) {
            return false;
        }
        if (empty(Country::getIsoById($address->id_country))) {
            return false;
        }
        $addressCountryIso = Country::getIsoById($address->id_country);
        return self::getApiDataByCountryCode($addressCountryIso);
    }

    public static function getApiDataByCountryCode($countryIso)
    {
        if (empty($countryIso)) {
            return false;
        }
        $defaultCountry = Configuration::get('ESTO_COUNTRY');
        $defaultCountryIso = self::API_COUNTRIES[$defaultCountry];
        if ($countryIso == $defaultCountryIso) {
            #pass
        } elseif (!empty(Configuration::get('ESTO_USE_SPECIFIC_ENDPOINT_'.$countryIso))) {
            #pass
        } else {
            return false; # address country is not in the list
        }
        if (empty(self::ISO_COUNTRIES[$countryIso])) {
            return false;
        }
        $moduleCountryName = self::ISO_COUNTRIES[$countryIso];
        $api_details = self::getCountryApiData($moduleCountryName);
        return $api_details;
    }

    protected function validateCredentials($countryName)
    {
        if (array_key_exists($countryName, self::SUBSTITUTED_COUNTRIES)) {
            $countryName = self::SUBSTITUTED_COUNTRIES[$countryName];
        }
        $country_iso = self::API_COUNTRIES[$countryName];
        if (empty(strval(Tools::getValue('ESTO_SHOP_ID_'.$country_iso)))){
            return false;
        }
        if (empty(strval(Tools::getValue('ESTO_SECRET_KEY_'.$country_iso)))){
            return false;
        }
        return true;
    }

    protected function validateSpecificCountryCredentials($ee_used, $lv_used, $lt_used)
    {
        $validationResults = [];
        if ($ee_used) {
            $validationResults[] = $this->validateCredentials(self::COUNTRY_ESTONIA);
        }
        if ($lv_used) {
            $validationResults[] = $this->validateCredentials(self::COUNTRY_LATVIA);
        }
        if ($lt_used) {
            $validationResults[] = $this->validateCredentials(self::COUNTRY_LITHUANIA);
        }
        foreach ($validationResults as $valid) {
            if (!$valid) {
                return false;
            }
        }
        return true;
    }

    public function hookActionFrontControllerSetMedia()
    {
        if ('order' == Tools::getValue('controller')){
            $this->context->controller->registerJavascript(
                'esto-javascript',
                $this->_path.'views/js/esto.js',
                [
                    'position' => 'bottom',
                    'priority' => 1000,
                ]
            );
        }
    }

    protected function getFirstCardOption($data_banks, $country_iso)
    {
        if (!isset($data_banks[$country_iso])) {
            return false;
        }

        foreach ($data_banks[$country_iso] as $key => $option) {
            if (self::CARD_TYPE_NAME !== $option['type']) {
                continue;
            }
            return $option;
        }

        return false;
    }

    protected function getEstoXPayments($total)
    {
        $estoXParts = 3;
        $monthNumericFormat = 'n';
        $months = [
            '1' => $this->l('January'),
            '2' => $this->l('February'),
            '3' => $this->l('March'),
            '4' => $this->l('April'),
            '5' => $this->l('May'),
            '6' => $this->l('June'),
            '7' => $this->l('July'),
            '8' => $this->l('August'),
            '9' => $this->l('September'),
            '10' => $this->l('October'),
            '11' => $this->l('November'),
            '12' => $this->l('December'),
        ];

        $total = (int)round(($total*100));
        $extraCents = $total % $estoXParts;
        $paymentBase = floor ($total/$estoXParts);

        if (empty($paymentBase)) {
            return [];
        }

        $payments = [];
        $dateTime = new DateTime("now");
        $one_month = new DateInterval('P1M');

        for ($i = 1; $i <= $estoXParts; $i++) {
            $dateTime->add($one_month);
            $amount = $paymentBase;
            if ($i <= $extraCents){
                $amount++;
            }
            $amountFormatted = Tools::displayPrice($amount/100);
            $payments[] = ['amount'=> $amountFormatted, 'month' => $months[$dateTime->format($monthNumericFormat)]];
        }

        return $payments;
    }

    public function getBankPaymentSuffix($bankName, $idCart)
    {
        $idCart = (int)$idCart;
        if (empty($bankName) || !$idCart){
            return '';
        }

        $bankName = mb_strtoupper($bankName);
        $bankPaymentSuffix = sprintf(' - %s (Cart ID %s)', $bankName, $idCart);

        return $bankPaymentSuffix;
    }
}
